@@ -18,13 +18,16 @@
 #include <skiboot.h>
 #include <chip.h>
 #include <device.h>
+#include <timebase.h>
 
 static struct proc_chip *chips[MAX_CHIPS];
 enum proc_chip_quirks proc_chip_quirks;
 
 uint32_t pir_to_chip_id(uint32_t pir)
 {
-	if (proc_gen == proc_gen_p8)
+	if (proc_gen == proc_gen_p9)
+		return P9_PIR2GCID(pir);
+	else if (proc_gen == proc_gen_p8)
 		return P8_PIR2GCID(pir);
 	else
 		return P7_PIR2GCID(pir);
@@ -32,7 +35,9 @@ uint32_t pir_to_chip_id(uint32_t pir)
 
 uint32_t pir_to_core_id(uint32_t pir)
 {
-	if (proc_gen == proc_gen_p8)
+	if (proc_gen == proc_gen_p9)
+		return P9_PIR2COREID(pir);
+	else if (proc_gen == proc_gen_p8)
 		return P8_PIR2COREID(pir);
 	else
 		return P7_PIR2COREID(pir);
@@ -40,7 +45,9 @@ uint32_t pir_to_core_id(uint32_t pir)
 
 uint32_t pir_to_thread_id(uint32_t pir)
 {
-	if (proc_gen == proc_gen_p8)
+	if (proc_gen == proc_gen_p9)
+		return P9_PIR2THREADID(pir);
+	else if (proc_gen == proc_gen_p8)
 		return P8_PIR2THREADID(pir);
 	else
 		return P7_PIR2THREADID(pir);
@@ -73,11 +80,27 @@ void init_chips(void)
 	if (dt_find_by_path(dt_root, "/mambo")) {
 		proc_chip_quirks |= QUIRK_NO_CHIPTOD | QUIRK_MAMBO_CALLOUTS
 			| QUIRK_NO_F000F | QUIRK_NO_PBA | QUIRK_NO_OCC_IRQ
-			| QUIRK_DISABLE_NAP;
+			| QUIRK_NO_DIRECT_CTL;
 		prlog(PR_NOTICE, "CHIP: Detected Mambo simulator\n");
 	}
+	/* Detect simics */
+	if (dt_find_by_path(dt_root, "/simics")) {
+		proc_chip_quirks |= QUIRK_SIMICS
+			| QUIRK_NO_PBA | QUIRK_NO_OCC_IRQ | QUIRK_SLOW_SIM;
+		tb_hz = 512000;
+		prlog(PR_NOTICE, "CHIP: Detected Simics simulator\n");
+	}
+	/* Detect Awan emulator */
+	if (dt_find_by_path(dt_root, "/awan")) {
+		proc_chip_quirks |= QUIRK_NO_CHIPTOD | QUIRK_NO_F000F
+			| QUIRK_NO_PBA | QUIRK_NO_OCC_IRQ | QUIRK_SLOW_SIM;
+		tb_hz = 512000;
+		prlog(PR_NOTICE, "CHIP: Detected Awan emulator\n");
+	}
+	/* Detect Qemu */
 	if (dt_node_is_compatible(dt_root, "qemu,powernv")) {
-		proc_chip_quirks |= QUIRK_NO_CHIPTOD | QUIRK_NO_PBA;
+		proc_chip_quirks |= QUIRK_NO_CHIPTOD | QUIRK_NO_PBA
+			| QUIRK_NO_DIRECT_CTL;
 		prlog(PR_NOTICE, "CHIP: Detected Qemu simulator\n");
 	}
 
@@ -96,7 +119,11 @@ void init_chips(void)
 						    0xffffffff);
 		chip->pcid = dt_prop_get_u32_def(xn, "ibm,proc-chip-id",
 						 0xffffffff);
+		if (dt_prop_get_u32_def(xn, "ibm,occ-functional-state", 1))
+			chip->occ_functional = true;
+		else
+			chip->occ_functional = false;
+
 		list_head_init(&chip->i2cms);
-		list_head_init(&chip->lpc_clients);
 	};
 }
