@@ -67,7 +67,7 @@ static int mambo_char = -1;
 static bool mambo_con_poll_read(void)
 {
 	if (mambo_char < 0)
-		mambo_char = mambo_read();
+		mambo_char = mambo_console_read();
 	return mambo_char >= 0;
 }
 
@@ -87,7 +87,7 @@ static size_t mambo_con_read(char *buf, size_t len)
 
 static size_t mambo_con_write(const char *buf, size_t len)
 {
-	mambo_write(buf, len);
+	mambo_console_write(buf, len);
 	return len;
 }
 
@@ -243,7 +243,7 @@ static size_t inmem_read(char *buf, size_t req)
 static void write_char(char c)
 {
 #ifdef MAMBO_DEBUG_CONSOLE
-	mambo_write(&c, 1);
+	mambo_console_write(&c, 1);
 #endif
 	inmem_write(c);
 }
@@ -338,6 +338,10 @@ static int64_t dummy_console_write(int64_t term_number, int64_t *length,
 {
 	if (term_number != 0)
 		return OPAL_PARAMETER;
+
+	if (!opal_addr_valid(length) || !opal_addr_valid(buffer))
+		return OPAL_PARAMETER;
+
 	write(0, buffer, *length);
 
 	return OPAL_SUCCESS;
@@ -349,6 +353,10 @@ static int64_t dummy_console_write_buffer_space(int64_t term_number,
 {
 	if (term_number != 0)
 		return OPAL_PARAMETER;
+
+	if (!opal_addr_valid(length))
+		return OPAL_PARAMETER;
+
 	if (length)
 		*length = INMEM_CON_OUT_LEN;
 
@@ -361,6 +369,10 @@ static int64_t dummy_console_read(int64_t term_number, int64_t *length,
 {
 	if (term_number != 0)
 		return OPAL_PARAMETER;
+
+	if (!opal_addr_valid(length) || !opal_addr_valid(buffer))
+		return OPAL_PARAMETER;
+
 	*length = read(0, buffer, *length);
 	opal_update_pending_evt(OPAL_EVENT_CONSOLE_INPUT, 0);
 
