@@ -366,14 +366,20 @@ static struct dt_property *new_property(struct dt_node *node,
 					const char *name, size_t size)
 {
 	struct dt_property *p = malloc(sizeof(*p) + size);
+	char *path;
+
 	if (!p) {
+		path = dt_get_path(node);
 		prerror("Failed to allocate property \"%s\" for %s of %zu bytes\n",
-			name, dt_get_path(node), size);
+			name, path, size);
+		free(path);
 		abort();
 	}
 	if (dt_find_property(node, name)) {
+		path = dt_get_path(node);
 		prerror("Duplicate property \"%s\" in node %s\n",
-			name, dt_get_path(node));
+			name, path);
+		free(path);
 		abort();
 
 	}
@@ -575,6 +581,14 @@ const struct dt_property *dt_find_property(const struct dt_node *node,
 	return NULL;
 }
 
+void dt_check_del_prop(struct dt_node *node, const char *name)
+{
+	struct dt_property *p;
+
+	p = __dt_find_property(node, name);
+	if (p)
+		dt_del_property(node, p);
+}
 const struct dt_property *dt_require_property(const struct dt_node *node,
 					      const char *name, int wanted_len)
 {
@@ -799,8 +813,6 @@ void dt_expand(const void *fdt)
 {
 	printf("FDT: Parsing fdt @%p\n", fdt);
 
-	dt_root = dt_new_root("");
-
 	if (dt_expand_node(dt_root, fdt, 0) < 0)
 		abort();
 }
@@ -904,3 +916,13 @@ u64 dt_translate_address(const struct dt_node *node, unsigned int index,
 	/* XXX TODO */
 	return dt_get_address(node, index, out_size);
 }
+
+bool dt_node_is_enabled(struct dt_node *node)
+{
+	const struct dt_property *p = dt_find_property(node, "status");
+
+	if (!p)
+		return true;
+
+	return p->len > 1 && p->prop[0] == 'o' && p->prop[1] == 'k';
+}
