@@ -18,6 +18,9 @@
 #include <timebase.h>
 #include <opal.h>
 #include <cpu.h>
+#include <chip.h>
+
+unsigned long tb_hz = 512000000;
 
 static void time_wait_poll(unsigned long duration)
 {
@@ -37,7 +40,8 @@ static void time_wait_poll(unsigned long duration)
 			opal_run_pollers();
 			time_wait_nopoll(period);
 			remaining -= period;
-		}
+		} else
+			time_wait_nopoll(remaining);
 
 		cpu_relax();
 	}
@@ -61,14 +65,27 @@ void time_wait(unsigned long duration)
 void time_wait_nopoll(unsigned long duration)
 {
 	unsigned long end = mftb() + duration;
+	unsigned long min = usecs_to_tb(10);
 
 	if (this_cpu()->tb_invalid) {
 		cpu_relax();
 		return;
 	}
 
-	while(tb_compare(mftb(), end) != TB_AAFTERB)
-		cpu_relax();
+	for (;;) {
+		uint64_t delay, tb = mftb();
+
+		if (tb_compare(tb, end) == TB_AAFTERB)
+			break;
+		delay = end - tb;
+		if (delay >= 0x7fffffff)
+			delay = 0x7fffffff;
+		if (delay >= min) {
+			mtspr(SPR_DEC, delay);
+			cpu_idle(cpu_wake_on_dec);
+		} else
+			cpu_relax();
+	}
 }
 
 void time_wait_ms(unsigned long ms)
@@ -108,7 +125,10 @@ unsigned long timespec_to_tb(const struct timespec *ts)
 	 * at the expense of capacity or do 128 bit math which
 	 * I'm not eager to do :-)
 	 */
-	return (ns * (tb_hz >> 24)) / (1000000000ul >> 24);
+	if (chip_quirk(QUIRK_SLOW_SIM))
+		return (ns * (tb_hz >> 16)) / (1000000000ul >> 16);
+	else
+		return (ns * (tb_hz >> 24)) / (1000000000ul >> 24);
 }
 
 int nanosleep(const struct timespec *req, struct timespec *rem)
