@@ -40,8 +40,8 @@ static struct {
 void init_boot_tracebuf(struct cpu_thread *boot_cpu)
 {
 	init_lock(&boot_tracebuf.trace_info.lock);
-	boot_tracebuf.trace_info.tb.mask = BOOT_TBUF_SZ - 1;
-	boot_tracebuf.trace_info.tb.max_size = MAX_SIZE;
+	boot_tracebuf.trace_info.tb.mask = cpu_to_be64(BOOT_TBUF_SZ - 1);
+	boot_tracebuf.trace_info.tb.max_size = cpu_to_be32(MAX_SIZE);
 
 	boot_cpu->trace = &boot_tracebuf.trace_info;
 }
@@ -103,7 +103,7 @@ static bool handle_repeat(struct tracebuf *tb, const union trace *trace)
 	rpt->type = TRACE_REPEAT;
 	rpt->len_div_8 = sizeof(*rpt) >> 3;
 	rpt->cpu = trace->hdr.cpu;
-	rpt->prev_len = trace->hdr.len_div_8 << 3;
+	rpt->prev_len = cpu_to_be16(trace->hdr.len_div_8 << 3);
 	rpt->num = cpu_to_be16(1);
 	lwsync(); /* write barrier: complete repeat record before exposing */
 	tb->end = cpu_to_be64(be64_to_cpu(tb->end) + sizeof(*rpt));
