@@ -24,6 +24,7 @@
 #include <sys/stat.h>
 #include <dirent.h>
 #include <limits.h>
+#include <inttypes.h>
 
 #include <ccan/array_size/array_size.h>
 
@@ -417,8 +418,8 @@ static int do_clear_i(struct gard_ctx *ctx, int pos, struct gard_record *gard, v
 		rc = blocklevel_smart_write(ctx->bl, buf_pos - sizeof_gard(ctx), buf, buf_len);
 		free(buf);
 		if (rc) {
-			fprintf(stderr, "Couldn't write to flash at 0x%08lx for len 0x%08x\n",
-			        buf_pos - sizeof_gard(ctx), buf_len);
+			fprintf(stderr, "Couldn't write to flash at 0x%08x for len 0x%08x\n",
+			        buf_pos - (int) sizeof_gard(ctx), buf_len);
 			return rc;
 		}
 	}
@@ -485,7 +486,7 @@ int check_gard_partition(struct gard_ctx *ctx)
 	if (ctx->gard_data_len == 0 || ctx->gard_data_len % sizeof(struct gard_record) != 0)
 		/* Just warn for now */
 		fprintf(stderr, "The %s partition doesn't appear to be an exact multiple of"
-				"gard records in size: %lu vs %u (or partition is zero in length)\n",
+				"gard records in size: %zd vs %u (or partition is zero in length)\n",
 				FLASH_GARD_PART, sizeof(struct gard_record), ctx->gard_data_len);
 
 	/*
@@ -549,7 +550,7 @@ static void usage(const char *progname)
 	fprintf(stderr, "-f --file <file>\n\tDon't search for MTD device,"
 	                " read from <file>.\n\n");
 	fprintf(stderr, "-p --part\n\tUsed in conjunction with -f to specify"
-	                "that just\n");
+	                " that just\n");
 	fprintf(stderr, "\tthe GUARD partition is in <file> and libffs\n");
 	fprintf(stderr, "\tshouldn't be used.\n\n");
 
@@ -575,6 +576,7 @@ int main(int argc, char **argv)
 	const char *action, *progname;
 	char *filename = NULL;
 	struct gard_ctx _ctx, *ctx;
+	uint64_t bl_size;
 	int rc, i = 0;
 	bool part = 0;
 	bool ecc = 0;
@@ -643,10 +645,18 @@ int main(int argc, char **argv)
 		goto out_free;
 	}
 
-	rc = blocklevel_get_info(ctx->bl, NULL, &(ctx->f_size), NULL);
+	rc = blocklevel_get_info(ctx->bl, NULL, &bl_size, NULL);
 	if (rc)
 		goto out;
 
+	if (bl_size > UINT_MAX) {
+		fprintf(stderr, "MTD device bigger than %i: size: %" PRIu64 "\n",
+			UINT_MAX, bl_size);
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+	ctx->f_size = bl_size;
+
 	if (!part) {
 		rc = ffs_init(0, ctx->f_size, ctx->bl, &ctx->ffs, 1);
 		if (rc)
