@@ -297,13 +297,14 @@ int hservice_scom_read(uint64_t chip_id, uint64_t addr, void *buf)
 				"failed: %m", chip_id, addr);
 		return 0;
 	}
+	rc = (int)scom.rc;
 
-	pr_debug("SCOM: read: chip 0x%lx, addr 0x%lx, val 0x%lx",
-			chip_id, addr, scom.data);
+	pr_debug("SCOM: read: chip 0x%lx, addr 0x%lx, val 0x%lx, rc %d",
+		 chip_id, addr, scom.data, rc);
 
 	*(uint64_t *)buf = htobe64(scom.data);
 
-	return 0;
+	return rc;
 }
 
 int hservice_scom_write(uint64_t chip_id, uint64_t addr,
@@ -322,11 +323,12 @@ int hservice_scom_write(uint64_t chip_id, uint64_t addr,
 				"failed: %m", chip_id, addr);
 		return 0;
 	}
+	rc = (int)scom.rc;
 
-	pr_debug("SCOM: write: chip 0x%lx, addr 0x%lx, val 0x%lx",
-			chip_id, addr, scom.data);
+	pr_debug("SCOM: write: chip 0x%lx, addr 0x%lx, val 0x%lx, rc %d",
+		 chip_id, addr, scom.data, rc);
 
-	return 0;
+	return rc;
 }
 
 uint64_t hservice_get_reserved_mem(const char *name, uint32_t instance)
@@ -649,6 +651,14 @@ int hservice_memory_error(uint64_t i_start_addr, uint64_t i_endAddr,
 	return 0;
 }
 
+uint64_t hservice_get_interface_capabilities(uint64_t set)
+{
+	if (set == HBRT_CAPS_SET1_OPAL)
+		return HBRT_CAPS_OPAL_HAS_XSCOM_RC;
+
+	return 0;
+}
+
 int hservices_init(struct opal_prd_ctx *ctx, void *code)
 {
 	uint64_t *s, *d;
@@ -1371,6 +1381,7 @@ static void handle_prd_control_run_cmd(struct control_msg *send_msg,
 static void handle_prd_control(struct opal_prd_ctx *ctx, int fd)
 {
 	struct control_msg msg, *recv_msg, *send_msg;
+	struct opal_prd_msg omsg;
 	bool enabled = false;
 	int rc, size;
 
@@ -1423,6 +1434,11 @@ static void handle_prd_control(struct opal_prd_ctx *ctx, int fd)
 		handle_prd_control_occ_actuation(send_msg, enabled);
 		break;
 	case CONTROL_MSG_TEMP_OCC_RESET:
+		omsg.hdr.type = OPAL_PRD_MSG_TYPE_OCC_RESET_NOTIFY;
+		omsg.hdr.size = htobe16(sizeof(omsg));
+		rc = write(ctx->fd, &omsg, sizeof(omsg));
+		if (rc != sizeof(omsg))
+			pr_log(LOG_WARNING, "FW: Failed to send OCC_RESET message: %m");
 		handle_prd_control_occ_reset(send_msg);
 		break;
 	case CONTROL_MSG_TEMP_OCC_ERROR:
