@@ -22,7 +22,6 @@
 #include <ccan/str/str.h>
 #include <ccan/array_size/array_size.h>
 #include <device.h>
-#include <p5ioc2.h>
 #include <p7ioc.h>
 #include <vpd.h>
 #include <inttypes.h>
@@ -48,9 +47,9 @@ static void io_add_common(struct dt_node *hn, const struct cechub_io_hub *hub)
 	 */
 	dt_add_property(hn, "ranges", NULL, 0);
 	dt_add_property_cells(hn, "ibm,gx-bar-1",
-			      hi32(hub->gx_ctrl_bar1), lo32(hub->gx_ctrl_bar1));
+			      hi32(be64_to_cpu(hub->gx_ctrl_bar1)), lo32(be64_to_cpu(hub->gx_ctrl_bar1)));
 	dt_add_property_cells(hn, "ibm,gx-bar-2",
-			      hi32(hub->gx_ctrl_bar2), lo32(hub->gx_ctrl_bar2));
+			      hi32(be64_to_cpu(hub->gx_ctrl_bar2)), lo32(be64_to_cpu(hub->gx_ctrl_bar2)));
 
 	/* Add presence detect if valid */
 	if (hub->flags & CECHUB_HUB_FLAG_FAB_BR0_PDT)
@@ -131,58 +130,6 @@ static void io_get_loc_code(const void *sp_iohubs, struct dt_node *hn, const cha
 	}
 }
 
-static struct dt_node *io_add_p5ioc2(const struct cechub_io_hub *hub,
-				     const void *sp_iohubs)
-{
-	struct dt_node *hn;
-	uint64_t reg[2];
-
-	const void *kwvpd;
-	unsigned int kwvpd_sz;
-
-	prlog(PR_DEBUG, "    GX#%d BUID_Ext = 0x%x\n",
-	      be32_to_cpu(hub->gx_index),
-	      be32_to_cpu(hub->buid_ext));
-	prlog(PR_DEBUG, "    GX BAR 0 = 0x%016"PRIx64"\n",
-	      be64_to_cpu(hub->gx_ctrl_bar0));
-	prlog(PR_DEBUG, "    GX BAR 1 = 0x%016"PRIx64"\n",
-	      be64_to_cpu(hub->gx_ctrl_bar1));
-	prlog(PR_DEBUG, "    GX BAR 2 = 0x%016"PRIx64"\n",
-	      be64_to_cpu(hub->gx_ctrl_bar2));
-	prlog(PR_DEBUG, "    GX BAR 3 = 0x%016"PRIx64"\n",
-	      be64_to_cpu(hub->gx_ctrl_bar3));
-	prlog(PR_DEBUG, "    GX BAR 4 = 0x%016"PRIx64"\n",
-	      be64_to_cpu(hub->gx_ctrl_bar4));
-
-	/* We assume SBAR == GX0 + some hard coded offset */
-	reg[0] = cleanup_addr(be64_to_cpu(hub->gx_ctrl_bar0) + P5IOC2_REGS_OFFSET);
-	reg[1] = 0x2000000;
-
-	hn = dt_new_addr(dt_root, "io-hub", reg[0]);
-	if (!hn)
-		return NULL;
-
-	dt_add_property(hn, "reg", reg, sizeof(reg));
-	dt_add_property_strings(hn, "compatible", "ibm,p5ioc2");
-
-	kwvpd = HDIF_get_idata(sp_iohubs, CECHUB_ASCII_KEYWORD_VPD, &kwvpd_sz);
-	if (kwvpd && kwvpd != sp_iohubs) {
-		/*
-		 * XX We don't know how to properly find the LXRn
-		 * record so for now we'll just try LXR0 and if not
-		 * found, we try LXR1
-		 */
-		if (!io_get_lx_info(kwvpd, kwvpd_sz, 0, hn))
-			io_get_lx_info(kwvpd, kwvpd_sz, 1, hn);
-	} else
-		prlog(PR_DEBUG, "CEC:     P5IOC2 Keywords not found.\n");
-
-	/* Get slots base loc code */
-	io_get_loc_code(sp_iohubs, hn, "ibm,io-base-loc-code");
-
-	return hn;
-}
-
 static struct dt_node *io_add_p7ioc(const struct cechub_io_hub *hub,
 				    const void *sp_iohubs)
 {
@@ -207,8 +154,8 @@ static struct dt_node *io_add_p7ioc(const struct cechub_io_hub *hub,
 	      be64_to_cpu(hub->gx_ctrl_bar4));
 
 	/* We only know about memory map 1 */
-	if (hub->mem_map_vers != 1) {
-		prerror("P7IOC: Unknown memory map %d\n", hub->mem_map_vers);
+	if (be32_to_cpu(hub->mem_map_vers) != 1) {
+		prerror("P7IOC: Unknown memory map %d\n", be32_to_cpu(hub->mem_map_vers));
 		/* We try to continue anyway ... */
 	}
 
@@ -354,7 +301,7 @@ static void io_add_p8_cec_vpd(const struct HDIF_common_hdr *sp_iohubs)
 		prlog(PR_WARNING, "CEC:     IOKID count is 0 !\n");
 		return;
 	}
-	if (iokids->count > 1) {
+	if (be32_to_cpu(iokids->count) > 1) {
 		prlog(PR_WARNING, "CEC:     WARNING ! More than 1 IO KID !!! (%d)\n",
 		      iokids->count);
 		/* Ignoring the additional ones */
@@ -379,213 +326,6 @@ static void io_add_p8_cec_vpd(const struct HDIF_common_hdr *sp_iohubs)
 	io_get_lx_info(kwvpd, kwvpd_sz, 0, dt_root);
 }
 
-static struct dt_node *io_add_hea(const struct cechub_io_hub *hub,
-				  const void *sp_io)
-{
-	struct dt_node *np, *gnp;
-	uint64_t reg[2];
-	unsigned int i, vpd_sz;
-	uint8_t kw_sz;
-	const void *iokid, *vpd, *ccin;
-	const uint8_t *mac;
-	const struct HDIF_child_ptr *iokids;
-
-	/*
-	 * We have a table of supported dauther cards looked up
-	 * by CCIN. We don't use the 1008 slot map in the VPD.
-	 *
-	 * This is basically translated from BML and will do for
-	 * now especially since we don't really support p5ioc2
-	 * machine, this is just for lab use
-	 *
-	 * This is mostly untested on 10G ... we might need more
-	 * info about the PHY in that case
-	 */
-	const struct hea_iocard {
-		const char ccin[4];
-		struct {
-			uint32_t speed;
-			uint16_t ports;
-			uint16_t phy_id;
-		} pg[2];
-	} hea_iocards[] = {
-		{
-			.ccin = "1818", /* HV4 something */
-			.pg[0] = { .speed = 1000, .ports = 2, .phy_id = 0 },
-		},
-		{
-			.ccin = "1819", /* HV4 Titov Card */
-			.pg[0] = { .speed = 1000, .ports = 2, .phy_id = 0 },
-			.pg[1] = { .speed = 1000, .ports = 2, .phy_id = 0 },
-		},
-		{
-			.ccin = "1830", /* HV4 Sergei Card */
-			.pg[0] = { .speed = 10000, .ports = 1, .phy_id = 0 },
-			.pg[1] = { .speed = 10000, .ports = 1, .phy_id = 0 },
-		},
-		{
-			.ccin = "181A", /* L4 Evans Card */
-			.pg[1] = { .speed = 1000, .ports = 2, .phy_id = 0 },
-		},
-		{
-			.ccin = "181B", /* L4 Weber Card */
-			.pg[0] = { .speed = 10000, .ports = 1, .phy_id = 0 },
-			.pg[1] = { .speed = 10000, .ports = 1, .phy_id = 0 },
-		},
-		{
-			.ccin = "181C", /* HV4 Gibson Card */
-			.pg[0] = { .speed = 1000, .ports = 2, .phy_id = 0 },
-			.pg[1] = { .speed = 1000, .ports = 2, .phy_id = 0 },
-		},
-		{
-			.ccin = "2BC4", /* MR Riverside 2 */
-			.pg[0] = { .speed = 1000, .ports = 1, .phy_id = 1 },
-			.pg[1] = { .speed = 1000, .ports = 1, .phy_id = 1 },
-		},
-		{
-			.ccin = "2BC5", /* MR Lions 2 */
-			.pg[0] = { .speed = 10000, .ports = 1, .phy_id = 1 },
-			.pg[1] = { .speed = 10000, .ports = 1, .phy_id = 1 },
-		},
-		{
-			.ccin = "2BC6", /* MR Onion 2 */
-			.pg[0] = { .speed = 10000, .ports = 1, .phy_id = 1 },
-			.pg[1] = { .speed = 1000, .ports = 2, .phy_id = 1 },
-		},
-		{
-			.ccin = "266D", /* Jupiter Bonzai */
-			.pg[0] = { .speed = 1000, .ports = 2, .phy_id = 1 },
-			.pg[1] = { .speed = 1000, .ports = 2, .phy_id = 1 },
-		},
-		/* The blade use an IO KID that's a bit oddball and seems to
-		 * represent the backplane itself, but let's use it anyway
-		 *
-		 * XXX Probably want a different PHY type !
-		 */
-		{
-			.ccin = "531C", /* P7 Blade */
-			.pg[0] = { .speed = 1000, .ports = 2, .phy_id = 0 },
-		},
-	};
-	const struct hea_iocard *card = NULL;
-
-	/* WARNING: This makes quite a lot of nasty assumptions
-	 * that appear to hold true on the few machines I care
-	 * about, which is good enough for now. We don't officially
-	 * support p5ioc2 anyway...
-	 */
-
-	/* Get first IO KID, we only support one. Real support would
-	 * mean using the FRU ID and the SLCA to find the right "stuff"
-	 * but at this stage it's unnecessary
-	 */
-	iokids = HDIF_child_arr(sp_io, CECHUB_CHILD_IO_KIDS);
-	if (!CHECK_SPPTR(iokids)) {
-		prerror("HEA: no IOKID in HDAT child array !\n");
-		return NULL;
-	}
-	if (!iokids->count) {
-		prerror("HEA: IOKID count is 0 !\n");
-		return NULL;
-	}
-	if (iokids->count > 1) {
-		prlog(PR_WARNING, "HEA: WARNING ! More than 1 IO KID !!! (%d)\n",
-		       iokids->count);
-	}
-	iokid = HDIF_child(sp_io, iokids, 0, "IO KID");
-	if (!iokid) {
-		prerror("HEA: Failed to retrieve IO KID 0 !\n");
-		return NULL;
-	}
-
-	/* Grab VPD */
-	vpd = HDIF_get_idata(iokid, IOKID_KW_VPD, &vpd_sz);
-	if (!CHECK_SPPTR(vpd)) {
-		prerror("HEA: Failed to retrieve VPD from IO KID !\n");
-		return NULL;
-	}
-
-	/* Grab the MAC address */
-	mac = vpd_find(vpd, vpd_sz, "VINI", "B1", &kw_sz);
-	if (!mac || kw_sz < 8) {
-		prerror("HEA: Failed to retrieve MAC Address !\n");
-		return NULL;
-	}
-
-	/* Grab the CCIN (card ID) */
-	ccin = vpd_find(vpd, vpd_sz, "VINI", "CC", &kw_sz);
-	if (!ccin || kw_sz < 4) {
-		prerror("HEA: Failed to retrieve CCIN !\n");
-		return NULL;
-	}
-
-	/* Now we could try to parse the 1008 slot map etc... but instead
-	 * we'll do like BML and grab the CCIN & use it for known cards.
-	 * We also grab the MAC
-	 */
-	for (i = 0; i < ARRAY_SIZE(hea_iocards) && !card; i++) {
-		if (strncmp(hea_iocards[i].ccin, ccin, 4))
-			continue;
-		card = &hea_iocards[i];
-	}
-	if (!card) {
-		prerror("HEA: Unknown CCIN 0x%.4s!\n", (const char *)ccin);
-		return NULL;
-	}
-
-	/* Assume base address is BAR3 + 0x4000000000 */
-	reg[0] = hub->gx_ctrl_bar3 + 0x4000000000;
-	reg[1] = 0xc0000000;
-
-	prlog(PR_DEBUG, "CEC:    * Adding HEA to P5IOC2, assuming GBA=0x%llx\n",
-	       (long long)reg[0]);
-	np = dt_new_addr(dt_root, "ibm,hea", reg[0]);
-	if (!np)
-		return NULL;
-
-	dt_add_property(np, "reg", reg, sizeof(reg));
-	dt_add_property_strings(np, "compatible", "ibm,p5ioc2-hea");
-	dt_add_property_cells(np, "#address-cells", 1);
-	dt_add_property_cells(np, "#size-cells", 0);
-	dt_add_property(np, "ibm,vpd", vpd, vpd_sz);
-	dt_add_property_cells(np, "#mac-address", mac[7]);
-	dt_add_property(np, "mac-address-base", mac, 6);
-	/* BUID is base + 0x30 */
-	dt_add_property(np, "interrupt-controller", NULL, 0);
-	dt_add_property_cells(np, "interrupt-base",
-			      ((hub->buid_ext << 9) | 0x30) << 4);
-	dt_add_property_cells(np, "interrupt-max-count", 128);
-
-	/* Always 2 port groups */
-	for (i = 0; i < 2; i++) {
-		unsigned int clause;
-
-		switch(card->pg[i].speed) {
-		case 1000:
-			clause = 0x22;
-			break;
-		case 10000:
-			clause = 0x45;
-			break;
-		default:
-			/* Unused port group */
-			continue;
-		}
-		gnp = dt_new_addr(np, "portgroup", i + 1);
-		if (!gnp)
-			continue;
-
-		dt_add_property_cells(gnp, "reg", i + 1);
-		dt_add_property_cells(gnp, "speed", card->pg[i].speed);
-		/* XX FIXME */
-		dt_add_property_strings(gnp, "phy-type", "mdio");
-		dt_add_property_cells(gnp, "phy-mdio-addr", card->pg[i].phy_id);
-		dt_add_property_cells(gnp, "phy-mdio-clause", clause);
-		dt_add_property_cells(gnp, "subports", card->pg[i].ports);
-	}
-	return np;
-}
-
 static void io_parse_fru(const void *sp_iohubs)
 {
 	unsigned int i;
@@ -643,12 +383,6 @@ static void io_parse_fru(const void *sp_iohubs)
 			hn = io_add_p7ioc(hub, sp_iohubs);
 			io_add_common(hn, hub);
 			break;
-		case CECHUB_HUB_P5IOC2:
-			prlog(PR_INFO, "CEC:     P5IOC2 !\n");
-			hn = io_add_p5ioc2(hub, sp_iohubs);
-			io_add_common(hn, hub);
-			io_add_hea(hub, sp_iohubs);
-			break;
 		case CECHUB_HUB_MURANO:
 		case CECHUB_HUB_MURANO_SEGU:
 			prlog(PR_INFO, "CEC:     Murano !\n");
