@@ -112,7 +112,7 @@ static struct dt_node *find_cpu_by_hardware_proc_id(struct dt_node *root,
 		if (!prop)
 			return NULL;
 
-		if (be32_to_cpu(*(u32 *)prop->prop) == hw_proc_id)
+		if (be32_to_cpu(*(be32 *)prop->prop) == hw_proc_id)
 			return i;
 	}
 	return NULL;
@@ -190,7 +190,6 @@ static void add_icps(void)
 				irange, sizeof(irange));
 		dt_add_property(icp, "interrupt-controller", NULL, 0);
 		dt_add_property_cells(icp, "#address-cells", 0);
-		dt_add_property_cells(icp, "#interrupt-cells", 1);
 		dt_add_property_string(icp, "device_type",
 				   "PowerPC-External-Interrupt-Presentation");
 		for (i = 0; i < num_threads*2; i += 2) {
@@ -327,10 +326,11 @@ static bool __paca_parse(void)
 	return true;
 }
 
-void paca_parse(void)
+int paca_parse(void)
 {
 	if (!__paca_parse()) {
 		prerror("CPU: Initial CPU parsing failed\n");
-		abort();
+		return -1;
 	}
+	return 0;
 }
