@@ -17,6 +17,8 @@
 #include <stdio.h>
 #include <stdarg.h>
 
+#include "../../ccan/list/list.c"
+
 void _prlog(int log_level __attribute__((unused)), const char* fmt, ...) __attribute__((format (printf, 2, 3)));
 
 #ifndef pr_fmt
@@ -29,7 +31,8 @@ void _prlog(int log_level __attribute__((unused)), const char* fmt, ...)
         va_list ap;
 
         va_start(ap, fmt);
-        vprintf(fmt, ap);
+        if (log_level < 7)
+		vprintf(fmt, ap);
         va_end(ap);
 }
 
@@ -42,25 +45,7 @@ static void stub_function(void)
 #define STUB(fnname) \
 	void fnname(void) __attribute__((weak, alias ("stub_function")))
 
-STUB(fdt_begin_node);
-STUB(fdt_property);
-STUB(fdt_end_node);
-STUB(fdt_create);
-STUB(fdt_add_reservemap_entry);
-STUB(fdt_finish_reservemap);
-STUB(fdt_strerror);
-STUB(fdt_check_header);
-STUB(_fdt_check_node_offset);
-STUB(fdt_next_tag);
-STUB(fdt_string);
-STUB(fdt_get_name);
-STUB(dt_first);
-STUB(dt_next);
-STUB(dt_has_node_property);
-STUB(dt_get_address);
 STUB(op_display);
 STUB(fsp_preload_lid);
 STUB(fsp_wait_lid_loaded);
-STUB(get_ics_phandle);
-STUB(get_psi_interrupt);
 STUB(fsp_adjust_lid_side);
