@@ -23,7 +23,6 @@
 #include <ccan/endian/endian.h>
 #include <errorlog.h>
 #include <opal-api.h>
-#include <fsp-elog.h>
 #include <timebase.h>
 
 #include "fsp-codeupdate.h"
@@ -1142,6 +1141,9 @@ static int64_t fsp_opal_update_flash(struct opal_sg_list *list)
 		rc = OPAL_SUCCESS;
 		goto out;
 	}
+
+	disable_fast_reboot("FSP Code Update");
+
 	length = (be64_to_cpu(list->length) & ~(SG_LIST_VERSION << 56)) - 16;
 	num_entries = length / sizeof(struct opal_sg_entry);
 	if (num_entries <= 0)
