@@ -1,4 +1,4 @@
-/* Copyright 2013-2014 IBM Corp.
+/* Copyright 2013-2016 IBM Corp.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,44 +16,40 @@
 
 
 /*
- * This code will enable retrieving of error log from fsp->sapphire
- * in sequence.
- * Here, FSP would send next log only when sapphire sends a new
- * log notification response to FSP. On Completion of reading
- * the log from FSP, OPAL_EVENT_ERROR_LOG_AVAIL is signaled.
- * This will remain raised until a call to opal_elog_read()
- * is made and OPAL_SUCCESS is returned, upon which.
- * the operation is complete and the event is cleared.
- * This is READ action from FSP.
+ * This code will enable retrieving of error log from FSP -> Sapphire in
+ * sequence.
+ * Here, FSP would send next log only when Sapphire sends a new log notification
+ * response to FSP. On Completion of reading the log from FSP,
+ * OPAL_EVENT_ERROR_LOG_AVAIL is signaled. This will remain raised until a call
+ * to opal_elog_read() is made and OPAL_SUCCESS is returned. Upon which, the
+ * operation is complete and the event is cleared. This is READ action from FSP.
  */
 
 /*
  * Design of READ error log :
- * When we receive a new error log entry notificatiion from FSP,
- * we queue it into the "pending" list.
- * If the "pending" list is not empty, then we start the fetching log from FSP.
+ * When we receive a new error log entry notification from FSP, we queue it into
+ * the "pending" list. If the "pending" list is not empty, then we start
+ * fetching log from FSP.
  *
- * When Linux reads a log entry, we dequeue it from the "pending" list
- * and enqueue it to another "processed" list. At this point, if the
- * "pending" list is not empty, we continue to fetch the next log.
+ * When Linux reads a log entry, we dequeue it from the "pending" list and
+ * enqueue it to another "processed" list. At this point, if the "pending"
+ * list is not empty, we continue to fetch the next log.
  *
- * When Linux calls opal_resend_pending_logs(), we fetch the log
- * corresponding to the head of the pending list and move it to the
- * processed list, and continue this process this until the pending list is
- * empty. If the pending list was empty earlier and is currently non-empty, we
- * initiate an error log fetch.
+ * When Linux calls opal_resend_pending_logs(), we fetch the log corresponding
+ * to the head of the pending list and move it to the processed list, and
+ * continue this process until the pending list is empty. If the pending list
+ * was empty earlier and is currently non-empty, we initiate an error log fetch.
  *
  * When Linux acks an error log, we remove it from processed list.
  */
 
-#include <skiboot.h>
-#include <fsp.h>
-#include <cpu.h>
-#include <lock.h>
 #include <errno.h>
-#include <psi.h>
+#include <fsp.h>
 #include <fsp-elog.h>
+#include <lock.h>
 #include <opal-api.h>
+#include <psi.h>
+#include <skiboot.h>
 
 /*
  * Maximum number of entries that are pre-allocated
@@ -61,7 +57,7 @@
  */
 #define ELOG_READ_MAX_RECORD		128
 
-/* structure to maintain log-id,log-size, pending and processed list */
+/* Structure to maintain log-id, log-size, pending and processed list. */
 struct fsp_log_entry {
 	uint32_t log_id;
 	size_t log_size;
@@ -71,46 +67,45 @@ struct fsp_log_entry {
 static LIST_HEAD(elog_read_pending);
 static LIST_HEAD(elog_read_processed);
 static LIST_HEAD(elog_read_free);
-
 /*
- * lock is used to protect overwriting of processed and pending list
- * and also used while updating state of each log
+ * Lock is used to protect overwriting of processed and pending list
+ * and also used while updating state of each log.
  */
 static struct lock elog_read_lock = LOCK_UNLOCKED;
 
-/* log buffer  to copy FSP log for READ */
-#define ELOG_READ_BUFFER_SIZE	0x00004000
+#define ELOG_READ_BUFFER_SIZE		0x00004000
+/* Log buffer to copy FSP log for read */
 static void *elog_read_buffer;
 static uint32_t elog_head_id;	/* FSP entry ID */
-static size_t elog_head_size;	/* actual FSP log size */
-static uint32_t elog_read_retries;	/* bad response status count */
+static size_t elog_head_size;	/* Actual FSP log size */
+static uint32_t elog_read_retries;	/* Bad response status count */
 
 /* Initialize the state of the log */
 static enum elog_head_state elog_read_from_fsp_head_state = ELOG_STATE_NONE;
 
-/* Need forward declaration because of Circular dependency */
+static bool elog_enabled = false;
+
+/* Need forward declaration because of circular dependency. */
 static void fsp_elog_queue_fetch(void);
 
 /*
- * check the response message for mbox acknowledgment
+ * Check the response message for mbox acknowledgement
  * command send to FSP.
  */
 static void fsp_elog_ack_complete(struct fsp_msg *msg)
 {
 	uint8_t val;
 
-	if (!msg->resp)
-		return;
 	val = (msg->resp->word1 >> 8) & 0xff;
 	if (val != 0)
-		prerror("ELOG: Acknowledgment error\n");
+		prerror("ELOG: Acknowledgement error\n");
+
 	fsp_freemsg(msg);
 }
 
-/* send Error Log PHYP Acknowledgment to FSP with entry ID */
+/* Send error log PHYP acknowledgement to FSP with entry ID. */
 static int64_t fsp_send_elog_ack(uint32_t log_id)
 {
-
 	struct fsp_msg *ack_msg;
 
 	ack_msg = fsp_mkmsg(FSP_CMD_ERRLOG_PHYP_ACK, 1, log_id);
@@ -118,20 +113,21 @@ static int64_t fsp_send_elog_ack(uint32_t log_id)
 		prerror("ELOG: Failed to allocate ack message\n");
 		return OPAL_INTERNAL_ERROR;
 	}
+
 	if (fsp_queue_msg(ack_msg, fsp_elog_ack_complete)) {
 		fsp_freemsg(ack_msg);
 		ack_msg = NULL;
 		prerror("ELOG: Error queueing elog ack complete\n");
 		return OPAL_INTERNAL_ERROR;
 	}
+
 	return OPAL_SUCCESS;
 }
 
-/* retrive error log from FSP with TCE for the data transfer */
+/* Retrieve error log from FSP with TCE for the data transfer. */
 static void fsp_elog_check_and_fetch_head(void)
 {
 	lock(&elog_read_lock);
-
 	if (elog_read_from_fsp_head_state != ELOG_STATE_NONE ||
 			list_empty(&elog_read_pending)) {
 		unlock(&elog_read_lock);
@@ -139,42 +135,60 @@ static void fsp_elog_check_and_fetch_head(void)
 	}
 
 	elog_read_retries = 0;
-
 	/* Start fetching first entry from the pending list */
 	fsp_elog_queue_fetch();
 	unlock(&elog_read_lock);
 }
 
-/* this function should be called with the lock held */
-static void fsp_elog_set_head_state(enum elog_head_state state)
+void elog_set_head_state(bool opal_logs, enum elog_head_state state)
 {
-	enum elog_head_state old_state = elog_read_from_fsp_head_state;
+	static enum elog_head_state opal_logs_state = ELOG_STATE_NONE;
+	static enum elog_head_state fsp_logs_state = ELOG_STATE_NONE;
 
-	elog_read_from_fsp_head_state = state;
+	/* ELOG disabled */
+	if (!elog_enabled)
+		return;
 
-	if (state == ELOG_STATE_FETCHED_DATA &&
-			old_state != ELOG_STATE_FETCHED_DATA)
+	if (opal_logs)
+		opal_logs_state = state;
+	else
+		fsp_logs_state = state;
+
+	if (fsp_logs_state == ELOG_STATE_FETCHED_DATA ||
+		opal_logs_state == ELOG_STATE_FETCHED_DATA)
 		opal_update_pending_evt(OPAL_EVENT_ERROR_LOG_AVAIL,
 					OPAL_EVENT_ERROR_LOG_AVAIL);
-	if (state != ELOG_STATE_FETCHED_DATA &&
-			old_state == ELOG_STATE_FETCHED_DATA)
+	else
 		opal_update_pending_evt(OPAL_EVENT_ERROR_LOG_AVAIL, 0);
 }
 
+/* This function should be called with the lock held. */
+static inline void fsp_elog_set_head_state(enum elog_head_state state)
+{
+	elog_set_head_state(false, state);
+	elog_read_from_fsp_head_state = state;
+}
+
 /*
- * when we try maximum time of fetching log from fsp
+ * When, we try maximum time of fetching log from FSP
  * we call following function to delete log from the
- * pending list and update the state to fetch next log
+ * pending list and update the state to fetch next log.
  *
- * this function should be called with the lock held
+ * This function should be called with the lock held.
  */
 static void fsp_elog_fetch_failure(uint8_t fsp_status)
 {
 	struct fsp_log_entry *log_data;
 
-	/* read top list and delete the node */
+	/* Read top list and delete the node */
 	log_data = list_top(&elog_read_pending, struct fsp_log_entry, link);
 	if (!log_data) {
+		/**
+		 * @fwts-label ElogFetchFailureInconsistent
+		 * @fwts-advice Inconsistent state between OPAL and FSP
+		 * in code path for handling failure of fetching error log
+		 * from FSP. Likely a bug in interaction between FSP and OPAL.
+		 */
 		prlog(PR_ERR, "%s: Inconsistent internal list state !\n",
 		      __func__);
 	} else {
@@ -182,8 +196,8 @@ static void fsp_elog_fetch_failure(uint8_t fsp_status)
 		list_add(&elog_read_free, &log_data->link);
 		prerror("ELOG: received invalid data: %x FSP status: 0x%x\n",
 			log_data->log_id, fsp_status);
-
 	}
+
 	fsp_elog_set_head_state(ELOG_STATE_NONE);
 }
 
@@ -191,12 +205,10 @@ static void fsp_elog_fetch_failure(uint8_t fsp_status)
 static void fsp_elog_read_complete(struct fsp_msg *read_msg)
 {
 	uint8_t val;
-	/*struct fsp_log_entry *log_data;*/
 
 	lock(&elog_read_lock);
 	val = (read_msg->resp->word1 >> 8) & 0xff;
 	fsp_freemsg(read_msg);
-
 	if (elog_read_from_fsp_head_state == ELOG_STATE_REJECTED) {
 		fsp_elog_set_head_state(ELOG_STATE_NONE);
 		goto elog_read_out;
@@ -210,14 +222,15 @@ static void fsp_elog_read_complete(struct fsp_msg *read_msg)
 	case FSP_STATUS_DMA_ERROR:
 		if (elog_read_retries++ < MAX_RETRIES) {
 			/*
-			 * for a error response value from FSP, we try to
+			 * For a error response value from FSP, we try to
 			 * send fetch sp data mbox command again for three
 			 * times if response from FSP is still not valid
-			 * we send generic error response to fsp.
+			 * we send generic error response to FSP.
 			 */
 			fsp_elog_queue_fetch();
 			break;
 		}
+
 		fsp_elog_fetch_failure(val);
 		break;
 
@@ -232,7 +245,7 @@ elog_read_out:
 	fsp_elog_check_and_fetch_head();
 }
 
-/* read error log from FSP through mbox commands */
+/* Read error log from FSP through mbox commands */
 static void fsp_elog_queue_fetch(void)
 {
 	int rc;
@@ -241,15 +254,21 @@ static void fsp_elog_queue_fetch(void)
 
 	entry = list_top(&elog_read_pending, struct fsp_log_entry, link);
 	if (!entry) {
+		/**
+		 * @fwts-label ElogQueueInconsistent
+		 * @fwts-advice Bug in interaction between FSP and OPAL. We
+		 * expected there to be a pending read from FSP but the list
+		 * was empty.
+		 */
 		prlog(PR_ERR, "%s: Inconsistent internal list state !\n",
-		      __func__);
+			__func__);
 		fsp_elog_set_head_state(ELOG_STATE_NONE);
 		return;
 	}
+
 	fsp_elog_set_head_state(ELOG_STATE_FETCHING);
 	elog_head_id = entry->log_id;
 	elog_head_size = entry->log_size;
-
 	rc = fsp_fetch_data_queue(flags, FSP_DATASET_ERRLOG, elog_head_id,
 				  0, (void *)PSI_DMA_ERRLOG_READ_BUF,
 				  &elog_head_size, fsp_elog_read_complete);
@@ -259,13 +278,13 @@ static void fsp_elog_queue_fetch(void)
 	}
 }
 
-/* opal interface for powernv to read log size and log ID from sapphire */
+/* OPAL interface for PowerNV to read log size and log ID from Sapphire. */
 static int64_t fsp_opal_elog_info(uint64_t *opal_elog_id,
 				  uint64_t *opal_elog_size, uint64_t *elog_type)
 {
 	struct fsp_log_entry *log_data;
 
-	/* copy type of the error log */
+	/* Copy type of the error log */
 	*elog_type = ELOG_TYPE_PEL;
 
 	/* Check if any OPAL log needs to be reported to the host */
@@ -277,44 +296,60 @@ static int64_t fsp_opal_elog_info(uint64_t *opal_elog_id,
 		unlock(&elog_read_lock);
 		return OPAL_WRONG_STATE;
 	}
+
 	log_data = list_top(&elog_read_pending, struct fsp_log_entry, link);
 	if (!log_data) {
+		/**
+		 * @fwts-label ElogInfoInconsistentState
+		 * @fwts-advice We expected there to be an entry in the list
+		 * of error logs for the error log we're fetching information
+		 * for. There wasn't. This means there's a bug.
+		 */
 		prlog(PR_ERR, "%s: Inconsistent internal list state !\n",
 		      __func__);
+		fsp_elog_set_head_state(ELOG_STATE_NONE);
 		unlock(&elog_read_lock);
 		return OPAL_WRONG_STATE;
 	}
+
 	*opal_elog_id = log_data->log_id;
 	*opal_elog_size = log_data->log_size;
+	fsp_elog_set_head_state(ELOG_STATE_HOST_INFO);
 	unlock(&elog_read_lock);
 	return OPAL_SUCCESS;
 }
 
-/* opal interface for powernv to read log from sapphire */
+/* OPAL interface for PowerNV to read log from Sapphire. */
 static int64_t fsp_opal_elog_read(uint64_t *buffer, uint64_t opal_elog_size,
 				  uint64_t opal_elog_id)
 {
 	int size = opal_elog_size;
 	struct fsp_log_entry *log_data;
 
-
-	/* Check if any OPAL log needs to be reported to the host */
+	/* Check if any OPAL log needs to be reported to the PowerNV */
 	if (opal_elog_read(buffer, opal_elog_size, opal_elog_id))
 		return OPAL_SUCCESS;
+
 	/*
 	 * Read top entry from list.
-	 * as we know always top record of the list is fetched from FSP
+	 * As we know always top record of the list is fetched from FSP
 	 */
 	lock(&elog_read_lock);
-	if (elog_read_from_fsp_head_state != ELOG_STATE_FETCHED_DATA) {
+	if (elog_read_from_fsp_head_state != ELOG_STATE_HOST_INFO) {
 		unlock(&elog_read_lock);
 		return OPAL_WRONG_STATE;
 	}
 
 	log_data = list_top(&elog_read_pending, struct fsp_log_entry, link);
 	if (!log_data) {
+		/**
+		 * @fwts-label ElogReadInconsistentState
+		 * @fwts-advice Inconsistent state while reading error log
+		 * from FSP. Bug in OPAL and FSP interaction.
+		 */
 		prlog(PR_ERR, "%s: Inconsistent internal list state !\n",
 		      __func__);
+		fsp_elog_set_head_state(ELOG_STATE_NONE);
 		unlock(&elog_read_lock);
 		return OPAL_WRONG_STATE;
 	}
@@ -333,32 +368,31 @@ static int64_t fsp_opal_elog_read(uint64_t *buffer, uint64_t opal_elog_size,
 	memcpy((void *)buffer, elog_read_buffer, size);
 
 	/*
-	 * once log is read from linux move record from pending
+	 * Once log is read from linux move record from pending
 	 * to processed list and delete record from pending list
-	 * and change state of the log to fetch next record
+	 * and change state of the log to fetch next record.
 	 */
 	list_del(&log_data->link);
 	list_add(&elog_read_processed, &log_data->link);
 	fsp_elog_set_head_state(ELOG_STATE_NONE);
 	unlock(&elog_read_lock);
 
-
-	/* read error log from FSP */
+	/* Read error log from FSP */
 	fsp_elog_check_and_fetch_head();
 
 	return OPAL_SUCCESS;
 }
 
-/* set state of the log head before fetching the log */
+/* Set state of the log head before fetching the log. */
 static void elog_reject_head(void)
 {
 	if (elog_read_from_fsp_head_state == ELOG_STATE_FETCHING)
 		fsp_elog_set_head_state(ELOG_STATE_REJECTED);
-	if (elog_read_from_fsp_head_state == ELOG_STATE_FETCHED_DATA)
+	else
 		fsp_elog_set_head_state(ELOG_STATE_NONE);
 }
 
-/* opal Interface for powernv to send ack to fsp with log ID */
+/* OPAL interface for PowerNV to send ack to FSP with log ID */
 static int64_t fsp_opal_elog_ack(uint64_t ack_id)
 {
 	int rc = 0;
@@ -373,33 +407,53 @@ static int64_t fsp_opal_elog_ack(uint64_t ack_id)
 		prerror("ELOG: failed to send acknowledgement: %d\n", rc);
 		return rc;
 	}
+
 	lock(&elog_read_lock);
-	list_for_each_safe(&elog_read_pending, record, next_record, link) {
-		if (record->log_id != ack_id)
-			continue;
-		list_del(&record->link);
-		list_add(&elog_read_free, &record->link);
-	}
 	list_for_each_safe(&elog_read_processed, record, next_record, link) {
 		if (record->log_id != ack_id)
 			continue;
+
 		list_del(&record->link);
 		list_add(&elog_read_free, &record->link);
+		unlock(&elog_read_lock);
+		return rc;
 	}
-	unlock(&elog_read_lock);
 
-	return rc;
+	list_for_each_safe(&elog_read_pending, record, next_record, link) {
+		if (record->log_id != ack_id)
+			continue;
+		/*
+		 * It means PowerNV has sent ACK without reading actual data.
+		 * Because of this elog_read_from_fsp_head_state may be
+		 * stuck in wrong state (ELOG_STATE_HOST_INFO) and not able
+		 * to send remaining ELOGs to PowerNV. Hence reset ELOG state
+		 * and start sending remaining ELOGs.
+		 */
+		list_del(&record->link);
+		list_add(&elog_read_free, &record->link);
+		elog_reject_head();
+		unlock(&elog_read_lock);
+		fsp_elog_check_and_fetch_head();
+		return rc;
+	}
+
+	unlock(&elog_read_lock);
+	return OPAL_PARAMETER;
 }
 
 /*
- * once linux kexec's it ask to resend all logs which
- * are not acknowledged from  linux
+ * Once Linux kexec's it ask to resend all logs which
+ * are not acknowledged from Linux.
  */
 static void fsp_opal_resend_pending_logs(void)
 {
 	struct fsp_log_entry  *entry;
 
-	/* Check if any Sapphire logs are pending */
+	lock(&elog_read_lock);
+	elog_enabled = true;
+	unlock(&elog_read_lock);
+
+	/* Check if any Sapphire logs are pending. */
 	opal_resend_pending_logs();
 
 	lock(&elog_read_lock);
@@ -414,24 +468,25 @@ static void fsp_opal_resend_pending_logs(void)
 		list_add(&elog_read_pending, &entry->link);
 	}
 
-	/*
-	 * If the current fetched or fetching log doesn't match our
-	 * new pending list head, then reject it
-	 */
-	if (!list_empty(&elog_read_pending)) {
-		entry = list_top(&elog_read_pending,
-					 struct fsp_log_entry, link);
-		if (entry->log_id != elog_head_id)
-			elog_reject_head();
-	}
-
 	unlock(&elog_read_lock);
 
-	/* Read error log from FSP if needed */
+	/* Read error log from FSP */
+	elog_reject_head();
 	fsp_elog_check_and_fetch_head();
 }
 
-/* fsp elog notify function  */
+/* Disable ELOG event flag until PowerNV is ready to receive event */
+static bool opal_kexec_elog_notify(void *data __unused)
+{
+	lock(&elog_read_lock);
+	elog_enabled = false;
+	opal_update_pending_evt(OPAL_EVENT_ERROR_LOG_AVAIL, 0);
+	unlock(&elog_read_lock);
+
+	return true;
+}
+
+/* FSP elog notify function */
 static bool fsp_elog_msg(uint32_t cmd_sub_mod, struct fsp_msg *msg)
 {
 	int rc = 0;
@@ -439,7 +494,6 @@ static bool fsp_elog_msg(uint32_t cmd_sub_mod, struct fsp_msg *msg)
 	uint32_t log_id;
 	uint32_t log_size;
 
-
 	if (cmd_sub_mod != FSP_CMD_ERRLOG_NOTIFICATION)
 		return false;
 
@@ -456,30 +510,30 @@ static bool fsp_elog_msg(uint32_t cmd_sub_mod, struct fsp_msg *msg)
 		       log_id, log_size);
 	}
 
-	/* take a lock until we take out the node from elog_read_free */
+	/* Take a lock until we take out the node from elog_read_free */
 	lock(&elog_read_lock);
 	if (!list_empty(&elog_read_free)) {
-		/* Create a new entry in the pending list */
+		/* Create a new entry in the pending list. */
 		record = list_pop(&elog_read_free, struct fsp_log_entry, link);
 		record->log_id = log_id;
 		record->log_size = log_size;
 		list_add_tail(&elog_read_pending, &record->link);
 		unlock(&elog_read_lock);
 
-		/* Send response back to FSP for a new elog notify message */
+		/* Send response back to FSP for a new elog notify message. */
 		rc = fsp_queue_msg(fsp_mkmsg(FSP_RSP_ERRLOG_NOTIFICATION,
 					1, log_id), fsp_freemsg);
 		if (rc)
 			prerror("ELOG: Failed to queue errlog notification"
 				" response: %d\n", rc);
 
-		/* read error log from FSP */
+		/* Read error log from FSP */
 		fsp_elog_check_and_fetch_head();
 
 	} else {
 		printf("ELOG: Log entry 0x%08x discarded\n", log_id);
 
-		/* unlock if elog_read_free is empty */
+		/* Unlock if elog_read_free is empty. */
 		unlock(&elog_read_lock);
 
 		rc = fsp_queue_msg(fsp_mkmsg(FSP_RSP_ERRLOG_NOTIFICATION,
@@ -487,9 +541,10 @@ static bool fsp_elog_msg(uint32_t cmd_sub_mod, struct fsp_msg *msg)
 		if (rc)
 			prerror("ELOG: Failed to queue errlog notification"
 				" response: %d\n", rc);
+
 		/*
-		 * if list is full with max record then we
-		 * send discarded by phyp (condition full) ack to FSP.
+		 * If list is full with max record then we send discarded by
+		 * phyp (condition full) ack to FSP.
 		 *
 		 * At some point in the future, we'll get notified again.
 		 * This is largely up to FSP as to when they tell us about
@@ -523,13 +578,14 @@ static int init_elog_read_free_list(uint32_t num_entries)
 		list_add_tail(&elog_read_free, &entry->link);
 		entry++;
 	}
+
 	return 0;
 
 out_err:
 	return -ENOMEM;
 }
 
-/* fsp elog read init function */
+/* FSP elog read init function */
 void fsp_elog_read_init(void)
 {
 	int val = 0;
@@ -547,15 +603,18 @@ void fsp_elog_read_init(void)
 	fsp_tce_map(PSI_DMA_ERRLOG_READ_BUF, elog_read_buffer,
 					PSI_DMA_ERRLOG_READ_BUF_SZ);
 
-	/* pre allocate memory for 128 record */
+	/* Pre allocate memory for 128 record */
 	val = init_elog_read_free_list(ELOG_READ_MAX_RECORD);
 	if (val != 0)
 		return;
 
-	/* register Eror log Class D2 */
+	/* Register error log class D2 */
 	fsp_register_client(&fsp_get_elog_notify, FSP_MCLASS_ERR_LOG);
 
-	/* register opal Interface */
+	/* Register for sync on PowerNV reboot call */
+	opal_add_host_sync_notifier(opal_kexec_elog_notify, NULL);
+
+	/* Register OPAL interface */
 	opal_register(OPAL_ELOG_READ, fsp_opal_elog_read, 3);
 	opal_register(OPAL_ELOG_ACK, fsp_opal_elog_ack, 1);
 	opal_register(OPAL_ELOG_RESEND, fsp_opal_resend_pending_logs, 0);
