@@ -249,8 +249,14 @@ struct fsp_msg *fsp_allocmsg(bool alloc_response)
 	msg = __fsp_allocmsg();
 	if (!msg)
 		return NULL;
-	if (alloc_response)
+	if (alloc_response) {
 		msg->resp = __fsp_allocmsg();
+		if (!msg->resp) {
+			free(msg);
+			return NULL;
+		}
+	}
+
 	return msg;
 }
 
@@ -1683,6 +1689,7 @@ void fsp_interrupt(void)
 	unlock(&fsp_lock);
 }
 
+
 int fsp_sync_msg(struct fsp_msg *msg, bool autofree)
 {
 	int rc;
@@ -1891,8 +1898,6 @@ static void fsp_update_links_states(struct fsp *fsp)
 	for (i = 0; i < fsp->iopath_count; i++) {
 		fiop = &fsp->iopath[i];
 		if (!fiop->psi)
-			continue;
-		if (!fiop->psi->working)
 			fiop->state = fsp_path_bad;
 		else if (fiop->psi->active) {
 			fsp->active_iopath = i;
@@ -1973,6 +1978,36 @@ static void fsp_opal_poll(void *data __unused)
 	}
 }
 
+int fsp_fatal_msg(struct fsp_msg *msg)
+{
+	int rc = 0;
+
+	rc = fsp_queue_msg(msg, NULL);
+	if (rc)
+		return rc;
+
+	while(fsp_msg_busy(msg)) {
+		cpu_relax();
+		fsp_opal_poll(NULL);
+	}
+
+	switch(msg->state) {
+	case fsp_msg_done:
+		rc = 0;
+		break;
+	case fsp_msg_timeout:
+		rc = -1; /* XXX to improve */
+		break;
+	default:
+		rc = -1; /* Should not happen... (assert ?) */
+	}
+
+	if (msg->resp)
+		rc = (msg->resp->word1 >> 8) & 0xff;
+
+	return rc;
+}
+
 static bool fsp_init_one(const char *compat)
 {
 	struct dt_node *fsp_node;
@@ -2127,7 +2162,7 @@ void fsp_opl(void)
 	fsp_sync_msg(fsp_mkmsg(FSP_CMD_CONTINUE_ACK, 0), true);
 
 	/* Wait for various FSP messages */
-	prlog(PR_INFO, "INIT: Waiting for FSP to advertize new role...\n");
+	prlog(PR_INFO, "INIT: Waiting for FSP to advertise new role...\n");
 	while(!(ipl_state & ipl_got_new_role)) {
 		cpu_relax();
 		opal_run_pollers();
@@ -2260,6 +2295,7 @@ int fsp_fetch_data_queue(uint8_t flags, uint16_t id, uint32_t sub_id,
 #define CAPP_IDX_VENICE_DD20 0x200ea
 #define CAPP_IDX_MURANO_DD20 0x200ef
 #define CAPP_IDX_MURANO_DD21 0x201ef
+#define CAPP_IDX_NAPLES_DD10 0x100d3
 
 static struct {
 	enum resource_id	id;
@@ -2272,6 +2308,7 @@ static struct {
 	{ RESOURCE_ID_CAPP,	CAPP_IDX_MURANO_DD21,	0x80a02001 },
 	{ RESOURCE_ID_CAPP,	CAPP_IDX_VENICE_DD10,	0x80a02003 },
 	{ RESOURCE_ID_CAPP,	CAPP_IDX_VENICE_DD20,	0x80a02004 },
+	{ RESOURCE_ID_CAPP,	CAPP_IDX_NAPLES_DD10,	0x80a02005 },
 };
 
 static void fsp_start_fetching_next_lid(void);
