@@ -19,13 +19,17 @@
 #include <opal.h>
 #include <skiboot.h>
 #include <string.h>
+#include <stdbool.h>
 
 #define IPMI_WRITE_SENSOR		(1 << 0)
 
 #define FW_PROGRESS_SENSOR_TYPE	0x0F
 #define BOOT_COUNT_SENSOR_TYPE	0xC3
 
-static int16_t sensors[255];
+#define MAX_IPMI_SENSORS 255
+static int16_t sensors[MAX_IPMI_SENSORS];
+
+static bool sensors_present = false;
 
 struct set_sensor_req {
 	u8 sensor_number;
@@ -36,8 +40,29 @@ struct set_sensor_req {
 	u8 event_data[3];
 };
 
+static bool ipmi_sensor_type_present(uint8_t sensor_type)
+{
+        const struct dt_property *type_prop;
+        uint8_t type;
+        struct dt_node *node;
+
+        dt_for_each_compatible(dt_root, node, "ibm,ipmi-sensor") {
+                type_prop = dt_find_property(node, "ipmi-sensor-type");
+                if (!type_prop) {
+                        prlog(PR_ERR, "IPMI: sensor doesn't have ipmi-sensor-type\n");
+                        continue;
+                }
+
+                type = (uint8_t)dt_property_get_cell(type_prop, 0);
+                if (type == sensor_type)
+                        return true;
+        }
+        return false;
+}
+
 uint8_t ipmi_get_sensor_number(uint8_t sensor_type)
 {
+	assert(sensor_type < MAX_IPMI_SENSORS);
 	return sensors[sensor_type];
 }
 
@@ -45,11 +70,19 @@ int ipmi_set_boot_count(void)
 {
 	struct set_sensor_req req;
 	struct ipmi_msg *msg;
-	int boot_count_sensor = sensors[BOOT_COUNT_SENSOR_TYPE];
+	int boot_count_sensor;
+
+	if (!sensors_present)
+		return OPAL_UNSUPPORTED;
 
 	if (!ipmi_present())
 		return OPAL_CLOSED;
 
+        if (!ipmi_sensor_type_present(BOOT_COUNT_SENSOR_TYPE))
+                return OPAL_HARDWARE;
+
+	boot_count_sensor = sensors[BOOT_COUNT_SENSOR_TYPE];
+
 	if (boot_count_sensor < 0) {
 		prlog(PR_DEBUG, "IPMI: boot count set but not present\n");
 		return OPAL_HARDWARE;
@@ -75,11 +108,19 @@ int ipmi_set_fw_progress_sensor(uint8_t state)
 {
 	struct ipmi_msg *msg;
 	struct set_sensor_req request;
-	int fw_sensor_num = sensors[FW_PROGRESS_SENSOR_TYPE];
+	int fw_sensor_num;
+
+	if (!sensors_present)
+		return OPAL_UNSUPPORTED;
 
 	if (!ipmi_present())
 		return OPAL_CLOSED;
 
+        if (!ipmi_sensor_type_present(FW_PROGRESS_SENSOR_TYPE))
+                return OPAL_HARDWARE;
+
+	fw_sensor_num = sensors[FW_PROGRESS_SENSOR_TYPE];
+
 	if (fw_sensor_num < 0) {
 		prlog(PR_DEBUG, "IPMI: fw progress set but not present\n");
 		return OPAL_HARDWARE;
@@ -90,6 +131,7 @@ int ipmi_set_fw_progress_sensor(uint8_t state)
 	request.sensor_number = fw_sensor_num;
 	request.operation = 0xa0; /* Set event data bytes, assertion bits */
 	request.assertion_mask[0] = 0x04; /* Firmware progress offset */
+	request.event_data[0] = 0xc2;
 	request.event_data[1] = state;
 
 	prlog(PR_INFO, "IPMI: setting fw progress sensor %02x to %02x\n",
@@ -125,6 +167,8 @@ void ipmi_sensor_init(void)
 		}
 		num = (uint8_t)dt_property_get_cell(num_prop, 0);
 		type = (uint8_t)dt_property_get_cell(type_prop, 0);
+		assert(type < MAX_IPMI_SENSORS);
 		sensors[type] = num;
 	}
+	sensors_present = true;
 }
