@@ -64,8 +64,8 @@ extern void set_console(struct con_ops *driver);
 
 extern void console_complete_flush(void);
 
-extern int mambo_read(void);
-extern void mambo_write(const char *buf, size_t count);
+extern int mambo_console_read(void);
+extern void mambo_console_write(const char *buf, size_t count);
 extern void enable_mambo_console(void);
 
 ssize_t console_write(bool flush_to_drivers, const void *buf, size_t count);
