@@ -697,6 +697,13 @@ extern void fsp_cancelmsg(struct fsp_msg *msg);
 extern int fsp_queue_msg(struct fsp_msg *msg,
 			 void (*comp)(struct fsp_msg *msg)) __warn_unused_result;
 
+/* Send a fatal message to FSP
+ *
+ * This will *not* run pollers.
+ * Use only when attempting to get the word out about how we died.
+ */
+extern int fsp_fatal_msg(struct fsp_msg *msg);
+
 /* Synchronously send a command. If there's a response, the status is
  * returned as a positive number. A negative result means an error
  * sending the message.
