@@ -1,4 +1,4 @@
-/* Copyright 2013-2014 IBM Corp.
+/* Copyright 2013-2016 IBM Corp.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 /* Some fwd declarations for types used further down */
 struct phb;
 struct pci_device;
+struct pci_slot;
 struct errorlog;
 
 enum resource_id {
@@ -30,6 +31,17 @@ enum resource_id {
 #define RESOURCE_SUBID_NONE 0
 #define RESOURCE_SUBID_SUPPORTED 1
 
+struct bmc_platform {
+	const char *name;
+
+	/*
+	 * Map IPMI_OEM_X to vendor commands for this BMC
+	 * 0 = unimplimented
+	 */
+	uint32_t ipmi_oem_partial_add_esel;
+	uint32_t ipmi_oem_pnor_access_status;
+};
+
 /*
  * Each platform can provide a set of hooks
  * that can affect the generic code
@@ -37,6 +49,13 @@ enum resource_id {
 struct platform {
 	const char	*name;
 
+	/*
+	 * If BMC is constant, bmc platform specified here.
+	 * Platforms can also call set_bmc_platform() if BMC platform is
+	 * not a constant.
+	 */
+	const struct bmc_platform *bmc;
+
 	/*
 	 * Probe platform, return true on a match, called before
 	 * any allocation has been performed outside of the heap
@@ -158,6 +177,10 @@ struct platform {
 	 */
 	int64_t		(*sensor_read)(uint32_t sensor_hndl, int token,
 				       uint32_t *sensor_data);
+	/*
+	 * Return the heartbeat time
+	 */
+	int		(*heartbeat_time)(void);
 
 	/*
 	 * OPAL terminate
@@ -169,6 +192,9 @@ extern struct platform __platforms_start;
 extern struct platform __platforms_end;
 
 extern struct platform	platform;
+extern const struct bmc_platform *bmc_platform;
+
+extern bool manufacturing_mode;
 
 #define DECLARE_PLATFORM(name)\
 static const struct platform __used __section(".platforms") name ##_platform
@@ -182,6 +208,6 @@ extern int resource_loaded(enum resource_id id, uint32_t idx);
 
 extern int wait_for_resource_loaded(enum resource_id id, uint32_t idx);
 
-extern void mambo_sim_exit(void);
+extern void set_bmc_platform(const struct bmc_platform *bmc);
 
 #endif /* __PLATFORM_H */
