@@ -39,9 +39,8 @@
 #define MSR_LE		PPC_BIT(63)	/* Little Endian */
 
 /* PIR */
-#define SPR_PIR_P8_THREAD_MASK	0x0007	/* Mask of thread bits */
+#define SPR_PIR_P9_MASK		0x07ff	/* Mask of implemented bits */
 #define SPR_PIR_P8_MASK		0x1fff	/* Mask of implemented bits */
-#define SPR_PIR_P7_THREAD_MASK	0x0003	/* Mask of thread bits */
 #define SPR_PIR_P7_MASK		0x03ff	/* Mask of implemented bits */
 
 /* SPR register definitions */
@@ -68,6 +67,7 @@
 #define SPR_SPURR	0x134	/* RW: Scaled Processor Utilization Resource */
 #define SPR_PURR	0x135	/* RW: Processor Utilization Resource reg */
 #define SPR_HDEC	0x136	/* RW: Hypervisor Decrementer */
+#define SPR_HRMOR	0x139	/* RW: Hypervisor Real Mode Offset reg */
 #define SPR_HSRR0	0x13a	/* RW: HV Exception save/restore reg 0 */
 #define SPR_HSRR1	0x13b	/* RW: HV Exception save/restore reg 1 */
 #define SPR_TFMR	0x13d
@@ -97,6 +97,7 @@
 #define SPR_LPCR_P8_PECE2	PPC_BIT(49)   /* Wake on external interrupts */
 #define SPR_LPCR_P8_PECE3	PPC_BIT(50)   /* Wake on decrementer */
 #define SPR_LPCR_P8_PECE4	PPC_BIT(51)   /* Wake on MCs, HMIs, etc... */
+#define SPR_LPCR_P9_LD		PPC_BIT(46)   /* Large decrementer mode bit */
 
 
 /* Bits in TFMR - control bits */
@@ -163,8 +164,11 @@
 /* Bits in HID0 */
 #define SPR_HID0_POWER8_4LPARMODE	PPC_BIT(2)
 #define SPR_HID0_POWER8_2LPARMODE	PPC_BIT(6)
-#define SPR_HID0_HILE			PPC_BIT(19)
-#define SPR_HID0_ENABLE_ATTN		PPC_BIT(31)
+#define SPR_HID0_POWER8_DYNLPARDIS	PPC_BIT(15)
+#define SPR_HID0_POWER8_HILE		PPC_BIT(19)
+#define SPR_HID0_POWER9_HILE		PPC_BIT(4)
+#define SPR_HID0_POWER8_ENABLE_ATTN	PPC_BIT(31)
+#define SPR_HID0_POWER9_ENABLE_ATTN	PPC_BIT(3)
 
 /* PVR bits */
 #define SPR_PVR_TYPE			0xffff0000
@@ -181,6 +185,7 @@
 #define PVR_TYPE_P8E	0x004b /* Murano */
 #define PVR_TYPE_P8	0x004d /* Venice */
 #define PVR_TYPE_P8NVL	0x004c /* Naples */
+#define PVR_TYPE_P9	0x004e
 
 #ifdef __ASSEMBLY__
 
@@ -232,7 +237,8 @@ static inline void mtmsrd(unsigned long val, int l)
 	asm volatile("mtmsrd %0,%1" : : "r"(val), "i"(l) : "memory");
 }
 
-static inline unsigned long mfspr(unsigned int spr)
+static inline __attribute__((always_inline))
+unsigned long mfspr(const unsigned int spr)
 {
 	unsigned long val;
 
@@ -240,7 +246,8 @@ static inline unsigned long mfspr(unsigned int spr)
 	return val;
 }
 
-static inline void mtspr(unsigned int spr, unsigned long val)
+static inline __attribute__((always_inline))
+void mtspr(const unsigned int spr, unsigned long val)
 {
 	asm volatile("mtspr %0,%1" : : "i"(spr), "r"(val) : "memory");
 }
