@@ -1,6 +1,18 @@
 #!/bin/bash
 
-if test -d .git;
+usage() {
+	echo "$0 [ prefix ]"
+	echo -e "\t Optionally specify a prefix other than 'skiboot'"
+	echo
+}
+
+if [ "$1" = "-h" -o "$1" = "--help" ] ;
+then
+	usage
+	exit 1;
+fi
+
+if test -e .git || git rev-parse --is-inside-work-tree > /dev/null 2>&1;
 then
 	version=`git describe --exact-match 2>/dev/null`
 	if [ -z "$version" ];
@@ -27,12 +39,22 @@ then
 		version="$version-$diffsha"
 	fi
 
+	if [ $# -eq 1 ];
+	then
+		version=`echo $version | sed s/skiboot/$1/`
+	fi
+
 	echo $version
 else
 	if [ ! -z "$SKIBOOT_VERSION" ];
 	then
 		echo $SKIBOOT_VERSION
 	else
-		exit 1;
+		if [ ! -z "`cat .version`" ];
+		then
+			cat .version
+		else
+			exit 1;
+		fi
 	fi
 fi
