@@ -39,25 +39,25 @@
  * Refer to pci-slot.h for the default PCI state set
  * when you're going to change below values.
  */
-#define FIRENZE_PCI_SLOT_NORMAL			0x00000000
-#define FIRENZE_PCI_SLOT_HRESET			0x00000200
-#define   FIRENZE_PCI_SLOT_HRESET_START		0x00000201
-#define FIRENZE_PCI_SLOT_FRESET			0x00000300
-#define FIRENZE_PCI_SLOT_FRESET_START		0x00000301
-#define   FIRENZE_PCI_SLOT_FRESET_WAIT_RSP	0x00000302
-#define   FIRENZE_PCI_SLOT_FRESET_DELAY		0x00000303
-#define   FIRENZE_PCI_SLOT_FRESET_POWER_STATE	0x00000304
-#define   FIRENZE_PCI_SLOT_FRESET_POWER_OFF	0x00000305
-#define   FIRENZE_PCI_SLOT_FRESET_POWER_ON	0x00000306
-#define   FIRENZE_PCI_SLOT_PERST_DEASSERT	0x00000307
-#define   FIRENZE_PCI_SLOT_PERST_DELAY		0x00000308
-#define FIRENZE_PCI_SLOT_PFRESET		0x00000400
-#define   FIRENZE_PCI_SLOT_PFRESET_START	0x00000401
-#define FIRENZE_PCI_SLOT_GPOWER			0x00000600
-#define   FIRENZE_PCI_SLOT_GPOWER_START		0x00000601
-#define FIRENZE_PCI_SLOT_SPOWER			0x00000700
-#define   FIRENZE_PCI_SLOT_SPOWER_START		0x00000701
-#define   FIRENZE_PCI_SLOT_SPOWER_DONE		0x00000702
+#define FIRENZE_PCI_SLOT_NORMAL			PCI_SLOT_STATE_NORMAL
+#define FIRENZE_PCI_SLOT_LINK			PCI_SLOT_STATE_LINK
+#define   FIRENZE_PCI_SLOT_LINK_START		(FIRENZE_PCI_SLOT_LINK + 1)
+#define FIRENZE_PCI_SLOT_HRESET			PCI_SLOT_STATE_HRESET
+#define   FIRENZE_PCI_SLOT_HRESET_START		(FIRENZE_PCI_SLOT_HRESET + 1)
+#define FIRENZE_PCI_SLOT_FRESET			PCI_SLOT_STATE_FRESET
+#define   FIRENZE_PCI_SLOT_FRESET_START		(FIRENZE_PCI_SLOT_FRESET + 1)
+#define   FIRENZE_PCI_SLOT_FRESET_WAIT_RSP	(FIRENZE_PCI_SLOT_FRESET + 2)
+#define   FIRENZE_PCI_SLOT_FRESET_DELAY		(FIRENZE_PCI_SLOT_FRESET + 3)
+#define   FIRENZE_PCI_SLOT_FRESET_POWER_STATE	(FIRENZE_PCI_SLOT_FRESET + 4)
+#define   FIRENZE_PCI_SLOT_FRESET_POWER_OFF	(FIRENZE_PCI_SLOT_FRESET + 5)
+#define   FIRENZE_PCI_SLOT_FRESET_POWER_ON	(FIRENZE_PCI_SLOT_FRESET + 6)
+#define   FIRENZE_PCI_SLOT_PERST_DEASSERT	(FIRENZE_PCI_SLOT_FRESET + 7)
+#define   FIRENZE_PCI_SLOT_PERST_DELAY		(FIRENZE_PCI_SLOT_FRESET + 8)
+#define FIRENZE_PCI_SLOT_GPOWER			PCI_SLOT_STATE_GPOWER
+#define   FIRENZE_PCI_SLOT_GPOWER_START		(FIRENZE_PCI_SLOT_GPOWER + 1)
+#define FIRENZE_PCI_SLOT_SPOWER			PCI_SLOT_STATE_SPOWER
+#define   FIRENZE_PCI_SLOT_SPOWER_START		(FIRENZE_PCI_SLOT_SPOWER + 1)
+#define   FIRENZE_PCI_SLOT_SPOWER_DONE		(FIRENZE_PCI_SLOT_SPOWER + 2)
 
 /* Timeout for power status */
 #define FIRENZE_PCI_SLOT_RETRIES	500
@@ -535,26 +535,8 @@ static int64_t firenze_pci_slot_freset(struct pci_slot *slot)
 		pval = (uint8_t *)(plat_slot->req->rw_buf);
 		*plat_slot->power_status = *pval;
 
-		/* PHB3 slot supports post fundamental reset, we switch
-		 * to that. For normal PCI slot, we switch to hot reset
-		 * instead.
-		 */
-		if (slot->ops.pfreset) {
-			prlog(PR_DEBUG, "%016llx FRESET: Switch to PFRESET\n",
-			      slot->id);
-			pci_slot_set_state(slot,
-				FIRENZE_PCI_SLOT_PFRESET_START);
-			return slot->ops.pfreset(slot);
-		} else if (slot->ops.hreset) {
-			prlog(PR_DEBUG, "%016llx FRESET: Switch to HRESET\n",
-			      slot->id);
-			pci_slot_set_state(slot,
-				FIRENZE_PCI_SLOT_HRESET_START);
-			return slot->ops.hreset(slot);
-		}
-
-		pci_slot_set_state(slot, FIRENZE_PCI_SLOT_NORMAL);
-		return OPAL_SUCCESS;
+		pci_slot_set_state(slot, FIRENZE_PCI_SLOT_LINK_START);
+		return slot->ops.poll_link(slot);
 	default:
 		prlog(PR_DEBUG, "%016llx FRESET: Unexpected state %08x\n",
 		      slot->id, slot->state);
@@ -615,27 +597,8 @@ static int64_t firenze_pci_slot_perst(struct pci_slot *slot)
 			FIRENZE_PCI_SLOT_PERST_DELAY);
 		return pci_slot_set_sm_timeout(slot, msecs_to_tb(1500));
 	case FIRENZE_PCI_SLOT_PERST_DELAY:
-		/*
-		 * Switch to post fundamental reset if the slot supports
-		 * that. Otherwise, we issue a proceeding hot reset on
-		 * the slot.
-		 */
-		if (slot->ops.pfreset) {
-			prlog(PR_DEBUG, "%016llx PERST: Switch to PFRESET\n",
-			      slot->id);
-			pci_slot_set_state(slot,
-				FIRENZE_PCI_SLOT_PFRESET_START);
-			return slot->ops.pfreset(slot);
-		} else if (slot->ops.hreset) {
-			prlog(PR_DEBUG, "%016llx PERST: Switch to HRESET\n",
-			      slot->id);
-			pci_slot_set_state(slot,
-				FIRENZE_PCI_SLOT_HRESET_START);
-			return slot->ops.hreset(slot);
-		}
-
-		pci_slot_set_state(slot, FIRENZE_PCI_SLOT_NORMAL);
-		return OPAL_SUCCESS;
+		pci_slot_set_state(slot, FIRENZE_PCI_SLOT_LINK_START);
+		return slot->ops.poll_link(slot);
 	default:
 		prlog(PR_DEBUG, "%016llx PERST: Unexpected state %08x\n",
 		      slot->id, slot->state);
@@ -688,6 +651,16 @@ static int64_t firenze_pci_slot_set_power_state(struct pci_slot *slot,
 	if (slot->power_state == val)
 		return OPAL_SUCCESS;
 
+	/* Update with the requested power state and bail immediately when
+	 * surprise hotplug is supported on the slot. It keeps the power
+	 * supply to the slot on and it guarentees the link state change
+	 * events will be raised properly during surprise hot add/remove.
+	 */
+	if (slot->surprise_pluggable) {
+		slot->power_state = val;
+		return OPAL_SUCCESS;
+	}
+
 	slot->power_state = val;
 	pci_slot_set_state(slot, FIRENZE_PCI_SLOT_SPOWER_START);
 
@@ -832,27 +805,12 @@ static void firenze_pci_slot_fixup(struct pci_slot *slot,
 	}
 }
 
-static void firenze_pci_slot_init(struct pci_slot *slot)
+static void firenze_pci_setup_power_mgt(struct pci_slot *slot,
+					struct firenze_pci_slot *plat_slot,
+					struct firenze_pci_slot_info *info)
 {
-	struct lxvpd_pci_slot *s = slot->data;
-	struct firenze_pci_slot *plat_slot = slot->data;
-	struct firenze_pci_slot_info *info = NULL;
-	uint32_t vdid;
 	uint8_t buddy;
-	int i;
 
-	/* Search for PCI slot info */
-	for (i = 0; i < ARRAY_SIZE(firenze_pci_slots); i++) {
-		if (firenze_pci_slots[i].index == s->slot_index &&
-		    !strcmp(firenze_pci_slots[i].label, s->label)) {
-			info = &firenze_pci_slots[i];
-			break;
-		}
-	}
-	if (!info)
-		return;
-
-	/* Search I2C bus for external power mgt */
 	buddy = info->buddy;
 	plat_slot->i2c_bus = firenze_pci_find_i2c_bus(info->chip_id,
 						      info->master_id,
@@ -891,6 +849,30 @@ static void firenze_pci_slot_init(struct pci_slot *slot)
 			plat_slot->i2c_bus = NULL;
 		}
 	}
+}
+
+static void firenze_pci_slot_init(struct pci_slot *slot)
+{
+	struct lxvpd_pci_slot *s = slot->data;
+	struct firenze_pci_slot *plat_slot = slot->data;
+	struct firenze_pci_slot_info *info = NULL;
+	uint32_t vdid;
+	int i;
+
+	/* Search for PCI slot info */
+	for (i = 0; i < ARRAY_SIZE(firenze_pci_slots); i++) {
+		if (firenze_pci_slots[i].index == s->slot_index &&
+		    !strcmp(firenze_pci_slots[i].label, s->label)) {
+			info = &firenze_pci_slots[i];
+			break;
+		}
+	}
+	if (!info)
+		return;
+
+	/* Search I2C bus for external power mgt */
+	if (slot->power_ctl)
+		firenze_pci_setup_power_mgt(slot, plat_slot, info);
 
 	/*
 	 * If the slot has external power logic, to override the
