@@ -30,4 +30,17 @@ extern int elog_fsp_commit(struct errorlog *buf);
 extern int64_t ibm_fsp_sensor_read(uint32_t sensor_hndl, int token,
 				uint32_t *sensor_data);
 
+/* Apollo PCI support */
+extern void apollo_pci_setup_phb(struct phb *phb,
+				 unsigned int index);
+extern void apollo_pci_get_slot_info(struct phb *phb,
+				     struct pci_device *pd);
+
+/* Firenze PCI support */
+extern void firenze_pci_send_inventory(void);
+extern void firenze_pci_setup_phb(struct phb *phb,
+				  unsigned int index);
+extern void firenze_pci_get_slot_info(struct phb *phb,
+				      struct pci_device *pd);
+
 #endif /*  __IBM_FSP_COMMON_H */
